name='altered.states'
version='0.8.6'
licence='ISC'
author='Jacob Oscarson'
author_email='jacob@plexical.com'
description = 'Monkey patching for Humans'
long_description = """
=================================
Python monkey-patching for Humans
=================================

*Altered States* tries to take the concept of "Python for Humans"
(http://python-for-humans.heroku.com/) to the world of monkey
patching.

Read more here:
https://github.com/Plexical/altered.states/blob/master/README.rst

Initial announcement:
http://www.plexical.com/blog/2012/03/06/altered-states/
"""
url = 'https://github.com/Plexical/altered.states'
